@@ -138,7 +138,7 @@ pub struct Processor{
     pub(crate) line_caches: HashMap<usize, String>,
     sandbox: bool,
     purge: bool,
-    strict: bool,
+    pub(crate) strict: bool,
     pub(crate) nopanic: bool,
     always_greedy: bool,
     // Temp target needs to save both path and file
@@ -539,19 +539,20 @@ impl Processor {
 
 
     /// Read from string
-    pub fn from_string(&mut self, content: &str) -> Result<String, RadError> {
+    pub fn from_string(&mut self, content: &str) -> Result<(), RadError> {
         // Set name as string
         self.set_input("String")?;
 
         let mut reader = content.as_bytes();
-        self.from_buffer(&mut reader, None)
+        self.from_buffer(&mut reader, None)?;
+        Ok(())
     }
 
     /// Read from standard input
     ///
     /// If debug mode is enabled this, doesn't read stdin line by line but by chunk because user
     /// input is also a standard input and processor cannot distinguish the two
-    pub fn from_stdin(&mut self) -> Result<String, RadError> {
+    pub fn from_stdin(&mut self) -> Result<(), RadError> {
         let stdin = io::stdin();
 
         // Early return if debug
@@ -561,15 +562,17 @@ impl Processor {
             let mut input = String::new();
             stdin.lock().read_to_string(&mut input)?;
             // This is necessary to prevent unexpected output from being captured.
-            return self.from_buffer(&mut input.as_bytes(), None);
+            self.from_buffer(&mut input.as_bytes(), None)?;
+            return Ok(());
         }
 
         let mut reader = stdin.lock();
-        self.from_buffer(&mut reader, None)
+        self.from_buffer(&mut reader, None)?;
+        Ok(())
     }
 
     /// Process contents from a file
-    pub fn from_file(&mut self, path :impl AsRef<Path>) -> Result<String, RadError> {
+    pub fn from_file(&mut self, path :impl AsRef<Path>) -> Result<(), RadError> {
         // Sandboxed environment, backup
         let backup = if self.sandbox { Some(self.backup()) } else { None };
         // Set file as name of given path
@@ -577,17 +580,15 @@ impl Processor {
 
         let file_stream = File::open(path)?;
         let mut reader = BufReader::new(file_stream);
-        self.from_buffer(&mut reader, backup)
+        self.from_buffer(&mut reader, backup)?;
+        Ok(())
     }
 
     /// Internal method for processing buffers line by line
-    fn from_buffer(&mut self,buffer: &mut impl std::io::BufRead, backup: Option<SandboxBackup>) -> Result<String, RadError> {
+    fn from_buffer(&mut self,buffer: &mut impl std::io::BufRead, backup: Option<SandboxBackup>) -> Result<(), RadError> {
         let mut line_iter = Utils::full_lines(buffer).peekable();
         let mut lexor = Lexor::new();
         let mut frag = MacroFragment::new();
-        let mut content = String::new();
-        // Container is where sandboxed output is saved
-        let mut container = if self.sandbox { Some(&mut content) } else { None };
         #[cfg(feature = "debug")]
         self.user_input_on_start()?;
         loop {
@@ -604,7 +605,7 @@ impl Processor {
                 // This means either macro is not found at all
                 // or previous macro fragment failed with invalid syntax
                 ParseResult::Printable(remainder) => {
-                    self.write_to(&remainder, &mut container)?;
+                    self.write_to(&remainder)?;
 
                     // Test if this works
                     #[cfg(feature = "debug")]
@@ -616,7 +617,7 @@ impl Processor {
                     }
                 }
                 ParseResult::FoundMacro(remainder) => {
-                    self.write_to(&remainder, &mut container)?;
+                    self.write_to(&remainder)?;
                 }
                 // This happens only when given macro involved text should not be printed
                 ParseResult::NoPrint => { }
@@ -637,7 +638,7 @@ impl Processor {
         if let Some(backup) = backup { self.recover(backup); self.sandbox = false; }
 
 
-        Ok(content)
+        Ok(())
     }
 
     // End of process methods
@@ -1121,7 +1122,9 @@ impl Processor {
         }
         // Find basic macro
         else if self.map.basic.contains(&name) {
-            let final_result = self.map.basic.clone().call(name, &args, greedy, self)?;
+            // Func always exists, because contains succeeded.
+            let func = self.map.basic.get(name).unwrap();
+            let final_result = func(&args, greedy, self)?;
             return Ok(EvalResult::Eval(final_result));
         } 
         // Find closure map
@@ -1180,7 +1183,7 @@ impl Processor {
     }
 
     /// Write text to either file or standard output according to processor's write option
-    fn write_to(&mut self, content: &str, container: &mut Option<&mut String>) -> Result<(), RadError> {
+    fn write_to(&mut self, content: &str) -> Result<(), RadError> {
         // Don't try to write empty string, because it's a waste
         if content.len() == 0 { return Ok(()); }
 
@@ -1189,21 +1192,14 @@ impl Processor {
         if self.yield_diff {
             self.diff_processed.as_ref().unwrap().write_all(content.as_bytes())?;
         }
-
-        // Save to container
-        if let Some(container) = container {
-            container.push_str(content);
-        } 
         // Write out to file or stdout
-        else {
-            if self.redirect {
-                self.temp_target.1.write(content.as_bytes())?;
-            } else {
-                match &mut self.write_option {
-                    WriteOption::File(f) => f.write_all(content.as_bytes())?,
-                    WriteOption::Terminal => print!("{}", content),
-                    WriteOption::Discard => () // Don't print anything
-                }
+        if self.redirect {
+            self.temp_target.1.write(content.as_bytes())?;
+        } else {
+            match &mut self.write_option {
+                WriteOption::File(f) => f.write_all(content.as_bytes())?,
+                WriteOption::Terminal => print!("{}", content),
+                WriteOption::Discard => () // Don't print anything
             }
         }
 
@@ -1313,8 +1309,49 @@ impl Processor {
                 }
             }
             frag.clear();
-        } 
-        else { // Invoke macro
+        } else if self.map.is_keyword(&frag.name) { // Is a keyword
+            let macro_func = self.map.keyword.get(&frag.name).unwrap();
+            let result = macro_func(&frag.args,level,self);
+
+            match result {
+                Ok(option) => {
+                    // Something to print
+                    if let Some(text) = option {
+                        self.write_to(&text)?;
+                    } else { // Nothing to print
+                        lexor.escape_nl = true;
+                    }
+
+                    // Clear fragment regardless of success
+                    frag.clear();
+                }
+                Err(err) => {
+                    self.log_error(&format!("{}", err))?;
+
+                    // If nopanic then don't panic
+                    if self.nopanic {
+                        // If purge mode is set, don't print anything 
+                        if !self.purge {
+                            remainder.push_str(&frag.whole_string);
+                        } else {
+                            // If purge mode
+                            // set escape new line 
+                            lexor.escape_nl = true;
+                        }
+
+                        // Clear fragment regardless of success
+                        frag.clear();
+
+                        return Ok(());
+                    } else {
+
+                        // Clear fragment regardless of success
+                        frag.clear();
+                        return Err(RadError::Panic);
+                    }
+                }
+            }
+        } else { // Invoke macro
             // Debug
             #[cfg(feature = "debug")]
             {
@@ -1355,7 +1392,13 @@ impl Processor {
         } else {
             self.log_error(&format!("{}", error))?;
         }
-        return Err(RadError::Panic);
+
+        // If nopanic don't panic
+        if self.nopanic {
+            Ok(())
+        } else {
+            Err(RadError::Panic)
+        }
     }
 
     fn lex_branch_end_frag_eval_result_ok(&mut self, variant : EvalResult, frag: &mut MacroFragment, remainder: &mut String, lexor : &mut Lexor, level: usize) -> Result<(), RadError> {
@@ -1501,6 +1544,9 @@ impl Processor {
         self.current_input = backup.current_input;
         self.map.local= backup.local_macro_map; 
         self.logger.recover_lines(backup.logger_lines);
+
+        // Also recover env values
+        self.set_file_env(&self.current_input);
     }
 
     /// Log error
@@ -1524,10 +1570,18 @@ impl Processor {
         } else {
             self.current_input = file.to_owned();
             self.logger.set_file(file);
+            self.set_file_env(file);
             Ok(())
         }
     }
 
+    /// Set some useful env values
+    fn set_file_env(&self, file: &str) {
+        let path = Path::new(file);
+        std::env::set_var("RAD_FILE", file);
+        std::env::set_var("RAD_FILE_DIR", path.parent().unwrap().to_str().unwrap());
+    }
+
     /// Set input as string not as &path
     /// 
     /// This is conceptualy identical to set_file but doesn't validate if given input is existent
